        // Run a game of noughts and crosses.
        // One side is taken by a human player.
class TTTInteractiveManager {
        // How big the game's board should be.
    public TTTInteractiveManager(int size){
        game = new TTTGame(size);
        // Create a display for the game.
        display = new TTTDisplay(size);
    }

        // Play a single game, interacting with a human opponent.
    public void play(){
        // The piece the human player will use.
        final char humanPiece = TTTGame.Cross;

        final TTTGame game = getGame();
        game.clearTheBoard();
        getDisplay().clear();

        // Who plays next.
        char player = TTTGame.Nought;
        while(game.gameState() == TTTGame.Unfinished){
            if(player == humanPiece){
                makeAHumanMove(player);
            }
            else{
                makeARandomMove(player);
            }
            if(!game.hasWon(player)){
                player = game.nextPlayer(player);
            }
        }
        reportTheResult();
    }

        // Make a random move for player.
    private void makeARandomMove(char player){
        final TTTGame game = getGame();
        // We shall generate a random row and column for each move.
        int row, col;
        // Generate a valid random move.
        do{
            row = getRandomPosition();
            col = getRandomPosition();
        } while(!game.isSquareFree(row,col));
        // Play in the chosen square.
        makeAMove(row,col,player);
    }

        // Ask the human player for a move.
    private void makeAHumanMove(char player){
        final TTTPlayer human = getHuman();
        final TTTGame game = getGame();
        int row, col;
        do{
            System.out.println("Type a row and column.");
            human.chooseSquare();
            // Retrieve the row and column chosen.
            row = human.getRow();
            col = human.getCol();
        } while(!game.isSquareFree(row,col));
        // Play in the chosen square.
        makeAMove(row,col,player);
    }

        // Implement the given move for player.
        // (row,col) is assumed to be free.
    private void makeAMove(int row, int col, char player){
        getGame().play(row,col,player);
        getDisplay().showSquare(row,col,player);
    }

    private void reportTheResult(){
        final TTTGame game = getGame();
        int result = game.gameState();

        if(result == TTTGame.Drawn){
            System.out.println("The game was drawn.");
        }
        else if(result == TTTGame.NoughtWon){
            System.out.println(TTTGame.Nought+" won.");
        }
        else if(result == TTTGame.CrossWon){
            System.out.println(TTTGame.Cross+" won.");
        }
        else{
            System.out.println("Unknown result: "+result);
        }
    }

    private TTTPlayer getHuman(){
        return human;
    }

        // Return a valid random number as part of a move.
    private int getRandomPosition(){
        return (int) (Math.random() * getGame().getBoardSize());
    }

    private TTTGame getGame(){
        return game;
    }

    private TTTDisplay getDisplay(){
        return display;
    }

    // The board. Initialized in the constructor.
    private final TTTGame game;
    // A GUI for the game.
    private final TTTDisplay display;
    // A human player.
    private final TTTPlayer human = new TTTPlayer();
}
