import java.io.*;
import java.util.LinkedList;

        // A class that provides operations on the lines of
        // a text file. In particular, operations associated
        // with detecting consecutive duplicate lines.
class DuplicateLines2 {
    public DuplicateLines2(String filename) throws IOException {
        BufferedReader reader =
                        new BufferedReader(new FileReader(filename));
        // Store the complete file in a LinkedList.
        LinkedList text = new LinkedList();
        String line = reader.readLine();
        while(line != null){
            text.add(line);
            line = reader.readLine();
        }
        // Turn text into an array.
        this.numLines = text.size();
        // Allocate space for an extra (null) String on the
        // end of the array. This will simplify handling of
        // the last line.
        this.lines = new String[numLines+1];
        text.toArray(lines);
        this.lines[numLines] = null;
        this.filename = filename;
    }

        // Write a new file containing only the non-duplicate lines.
    public void writeNonDuplicates(String outputfile) throws IOException {
        processLines(outputfile,true,false);
    }

        // Write a new file containing the non-duplicate lines and
        // a single copy of the duplicate lines.
    public void compressDuplicates(String outputfile) throws IOException {
        processLines(outputfile,true,true);
    }

        // Write a new file containing one copy of each of only the
        // duplicate lines.
    public void writeDuplicates(String outputfile) throws IOException {
        processLines(outputfile,false,true);
    }

    public String getFilename(){
        return filename;
    }

    public int getNumLines(){
        return numLines;
    }

        // Write a new file containing the non-duplicate lines and/or
        // duplicate lines, as required.
    private void processLines(String outputfile,
                             boolean nonDuplicatesRequired,
                             boolean duplicatesRequired)
                                        throws IOException {
        BufferedWriter writer = new BufferedWriter(
                                    new FileWriter(outputfile));
        final String[] lines = getLines();
        final int numLines = getNumLines();
        for(int lineNum = 0; lineNum < numLines; ){
            String nextLine = lines[lineNum];
            if(nextLine.equals(lines[lineNum+1])){
                // Duplicates.
                if(duplicatesRequired){
                    writer.write(nextLine);
                    writer.newLine();
                }
                do{ lineNum++;
                } while(lines[lineNum].equals(lines[lineNum+1]));
            }
            else{
                if(nonDuplicatesRequired){
                    writer.write(nextLine);
                    writer.newLine();
                }
            }
            lineNum++;
        }
        writer.close();
    }

    private String[] getLines(){
        return lines;
    }

    // Holds the full text of the file.
    private final String[] lines;
    // The number of lines in the input file. This does not
    // include the null (sentinel) added by the constructor.
    private final int numLines;
    // The name of the file.
    private final String filename;
}
