import java.io.*;
import java.util.StringTokenizer;

        // Supply numerical data from an input file representing
        // an encoded image. The data file consists of (count,color)
        // pairs. Comment lines start with #.
        //
        // A fresh StringTokenizer is created for each input line
        // containing image data. When one line is exhausted, the
        // next significant line is found and passed to a tokenizer.
        // The tokenizer delivers integers on successive calls
        // to nextInt.
class ImageReader {
    public static final char COMMENT_CHAR = '#';

    public ImageReader(String filename) throws IOException {
        this.reader = new BufferedReader(new FileReader(filename));
    }

        // Return the next integer value from the file.
    public int nextInt() throws IOException,
                                NumberFormatException {
        StringTokenizer tokenizer = getTokenizer();
        // See whether there is another number on the current line.
        if(!tokenizer.hasMoreTokens()){
            // Another line needs to be read.
            BufferedReader reader = getReader();
            String line = reader.readLine();
            // Find one that is significant.
            while(skipLine(line)){
                line = reader.readLine();
            }
            if(line != null){
                // A fresh tokenizer is needed.
                tokenizer = new StringTokenizer(line);
                if(tokenizer.hasMoreTokens()){
                    setTokenizer(tokenizer);
                }
                else{
                    throw new IOException("Malformed line: "+line);
                }
            }
            else{
                // Unexpected end of input.
                throw new IOException("End of input reached.");
            }
        }
        String token = tokenizer.nextToken();
        return Integer.parseInt(token);
    }

        // Return true if line is one that should be skipped, i.e.
        // it is blank or contains a comment. A null String should
        // not be skipped.
    private boolean skipLine(String line){
        if(line == null){
            return false;
        }
        else{
            line = line.trim();
            if(line.length() == 0){
                return true;
            }
            else{
                return line.charAt(0) == COMMENT_CHAR;
            }
        }
    }

    private BufferedReader getReader(){
        return reader;
    }

    private StringTokenizer getTokenizer(){
        return tokenizer;
    }

        // Switch to a fresh tokenizer for a new line.
    private void setTokenizer(StringTokenizer t){
        tokenizer = t;
    }

    private final BufferedReader reader;
    // The tokenizer operating on the current line of the file.
    private StringTokenizer tokenizer = new StringTokenizer("");
}
