        // Maintain a set of preset frequencies for a Radio.
        // The stationUp and stationDown methods provide
        // preset selection.
class PresetsRadio extends Radio {
    public PresetsRadio(double[] presets){
        setPresets(presets);
        if(presets.length > 0){
            // Select the first preset.
            setCurrentPreset(0);
        }
    }

        // Select the next preset station.
    public void stationUp() throws RuntimeException {
        int current = getCurrentPreset();
        setCurrentPreset(current+1);
    }

        // Select the previous preset station.
    public void stationDown() throws RuntimeException {
        int current = getCurrentPreset();
        setCurrentPreset(current-1);
    }

    public double[] getPresets(){
        return presets;
    }

    public void setPresets(double[] p){
        if(p != null){
            presets = p;
            // As the presets have been changed, reset
            // our idea of which will be the next station
            // selected.
            setCurrentPreset(0);
        }
    }

    private int getCurrentPreset(){
        return currentPreset;
    }

        // Select the requested preset frequency.
        // Implement wrap around at both ends of the presets array.
    private void setCurrentPreset(int p) throws RuntimeException {
        double[] presets = getPresets();
        final int numPresets = presets.length;
        if(numPresets == 0){
            throw new RuntimeException("No presets set.");
        }
        else{
            if(p < 0){
                p = numPresets-1;
            }
            else if(p >= numPresets){
                p = 0;
            }
            // else - within bounds already.
            currentPreset = p;
            setFrequency(presets[p]);
        }
    }

    private double[] presets = new double[0];
    // The current preset selected.
    private int currentPreset = -1;
}
