    // Maintain a stack of objects. This class provides a more limited
    // set of capabilities than java.util.LinkedList, for instance.
    // Use an inner class, Node, to provide the internal structure
    // of the stack.
public class Stack {
        // Add an object to the head of the stack.
    public void add(Object o){
        // Prefix the new node onto the existing head.
        Node n = new Node(o,getHead());
        // The new node becomes the new head.
        setHead(n);
    }

        // Return (and remove) the object at the head of the stack.
        // Return null if the stack is empty.
    public Object next(){
        Object item = null;
        Node head = getHead();
        if(head != null){
            item = head.getItem();
            // Drop the node from the head.
            setHead(head.getNext());
        }
        return item;
    }

        // Return a reference to the object at the head of the stack,
        // without removing it.
        // Return null if the stack is empty.
    public Object peek(){
        Object item = null;
        Node head = getHead();
        if(head != null){
            item = head.getItem();
        }
        return item;
    }

        // Return the length of the stack.
    public int size(){
        int length = 0;
        Node node = getHead();
        while(node != null){
            length++;
            node = node.getNext();
        }
        return length;
    }
        
        // Maintain a reference to a single item in the stack, and
        // a reference to the next in line.
    protected class Node {
        public Node(Object item,Node next){
            this.item = item;
            this.next = next;
        }

        public Object getItem(){
            return item;
        }

        public Node getNext(){
            return next;
        }

        public void setNext(Node n){
            next = n;
        }

        private final Object item;
        private Node next = null;
    }

    protected Node getHead(){
        return head;
    }

    protected void setHead(Node n){
        head = n;
    }

    // References to the node at the head of the stack.
    // The head is null when the stack is empty.
    private Node head = null;
}
