        // Simple definition of a SimpleShip class with position,
        // speed and course information.
        // This version incorporates changes as solutions to
        // several exercises. The moveTo method is new, and
        // both setSpeed and setCourse have additions.
public class SimpleShip {
    // Default position, course and speed.
    public static final double DEFAULT_X = 25.0, DEFAULT_Y = 25.0;
    public static final double DEFAULT_COURSE = 0.0, DEFAULT_SPEED = 5.0;
    // Define a maximum speed.
    public static final double MAXIMUM_SPEED = 10;

        // Constructors.
    public SimpleShip(double x, double y, double course, double speed){
        setPosition(x,y);
        setCourse(course);
        setSpeed(speed);
    }

    public SimpleShip(double x, double y, double course){
        this(x,y,course,DEFAULT_SPEED);
    }

    public SimpleShip(double x, double y){
        this(x,y,DEFAULT_COURSE,DEFAULT_SPEED);
    }

    public SimpleShip(){
        // Set default position, course and speed for this ship.
        this(DEFAULT_X,DEFAULT_Y,DEFAULT_COURSE,DEFAULT_SPEED);
    }

    public SimpleShip(String name){
        this();
        setName(name);
    }

        // Public mutators.
    public void setPosition(double x, double y){
        setX(x);
        setY(y);
    }

    public void setSpeed(double newSpeed){
        if(Math.abs(newSpeed) <= MAXIMUM_SPEED){
            speed = newSpeed;
        }
        else{
            // Use an appropriate maximum.
            if(newSpeed < 0){
                speed = -MAXIMUM_SPEED;
            }
            else{
                speed = MAXIMUM_SPEED;
            }
        }
    }

        // Set the course to newCourse. Only allow course changes
        // up to maxSingleChange degrees without a movement.
        // Implement any required movements.
    public void setCourse(double newCourse){
        // Keep degrees in the range [0..360)
        final double numDegrees = 360.0;
        // The maximum course change that may be made without
        // movement.
        final double maxSingleChange = 10.0;

        // Remove redundant multiples of 360 degrees.
        newCourse %= numDegrees;

        // Work out the minimum amount of course change required.
        double change = Math.abs(newCourse-course);
        if(change > numDegrees/2){
            // Adjust newCourse so that the change is < 180.
            newCourse += (newCourse < course)? numDegrees : -numDegrees;
            change = Math.abs(newCourse-course);
        }
        // Implement the full change in steps, with moves in between.
        while(change > maxSingleChange){
            if(course > newCourse){
                course -= maxSingleChange;
            }
            else{
                course += maxSingleChange;
            }
            move();
            change = Math.abs(newCourse-course);
        }
        // Set the final (positive) value of the course.
        if(newCourse < 0){
            newCourse = numDegrees+(newCourse%numDegrees);
        }
        else{
            newCourse %= numDegrees;
        }
        course = newCourse;
    }

        // Other public methods.
    public void report(){
        String name = getName();
        if((name != null) && !name.equals("")){
            System.out.print(name+": ");
        }
        System.out.print("Position is ("+getX()+","+getY()+"), ");
        System.out.print("Course is "+getCourse()+", ");
        System.out.print("Speed is "+getSpeed());
        System.out.println();
    }

    public void move(){
        double radians = Math.toRadians(getCourse());
        setPosition(getX()+speed*Math.cos(radians),
                    getY()+speed*Math.sin(radians));
    }

        // Move (close) to the given (x,y) position.
    public void moveTo(double targetX, double targetY){
        // Don't try to move closer than a unit movement at the
        // current speed.
        final double closeEnough = getSpeed();
        final double numDegrees = 360.0;
        double x = getX(), y = getY();
        while((Math.abs(targetX-x) >= closeEnough) ||
                          (Math.abs(targetY-y) >= closeEnough)){
            // Calculate the required course.
            double course;
            // Avoid division by zero.
            if(targetX != x){
                // Calculate the angle in the bottom right quadrant.
                double angleInRadians =
                    Math.atan(Math.abs((targetY-y)/(targetX-x)));
                course = Math.toDegrees(angleInRadians);
                // Adjust for the correct quadrant.
                if(targetX > x){
                    if(targetY < y){
                        course = numDegrees-course;
                    }
                    // else - no adjustment required.
                }
                else{
                    if(targetY < y){
                        course += (numDegrees/2);
                    }
                    else{
                        course = (numDegrees/2)-course;
                    }
                }
            }
            else{
                if(targetY < y){
                    course = 270;
                }
                else{
                    course = 90;
                }
            }
            setCourse(course);
            move();
            x = getX();
            y = getY();
        }
    }

        // Accessors to the basic attributes.
    public double getX(){
        return x;
    }

    public double getY(){
        return y;
    }

    public double getCourse(){
        return course;
    }

    public double getSpeed(){
        return speed;
    }

    public String getName(){
        return name;
    }

        // Private mutators.
    private void setX(double xval){
        x  = xval;
    }

    private void setY(double yval){
        y  = yval;
    }

    private void setName(String s){
        if(s != null){
            name = s;
        }
    }

    /* Private data. */
    // (x,y) co-ordinates in the ocean.
    private double x = DEFAULT_X, y = DEFAULT_Y;
    // Speed and course.
    private double speed = DEFAULT_SPEED, course = DEFAULT_COURSE;
    // A name for the ship.
    private String name = "";
}
